USE pubs
go

IF OBJECTPROPERTY(object_id(N'tsql_trycatch'), 'IsProcedure') = 1
DROP PROCEDURE tsql_trycatch1
go
CREATE PROCEDURE tsql_trycatch1
AS
BEGIN TRY
  BEGIN TRANSACTION;
	-- cause error 547
    DELETE jobs WHERE job_id = 3;
  COMMIT TRANSACTION;
END TRY
BEGIN CATCH 
	-- optional code here
	IF XACT_STATE() <> 0
		ROLLBACK TRANSACTION;
	-- EXECUTE usp_PrintError;
	EXECUTE usp_RethrowError; 
END CATCH;
go

EXEC tsql_trycatch1
SELECT @@error -- this equals 0

IF OBJECTPROPERTY(object_id(N'tsql_notry'), 'IsProcedure') = 1
DROP PROCEDURE tsql_notry
go

CREATE PROCEDURE tsql_notry
AS
	-- cause error 547
    DELETE jobs WHERE job_id = 3;
go


-- test the other one
EXEC tsql_notry
SELECT @@error -- this equals 547

IF OBJECTPROPERTY(object_id(N'tsql_error_wrapper1'), 'IsProcedure') = 1
DROP PROCEDURE tsql_error_wrapper1
go
CREATE PROCEDURE tsql_error_wrapper1
AS
BEGIN TRY
 EXEC tsql_trycatch1
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError;  
END CATCH
go

-- test it
EXEC tsql_error_wrapper1

-- name resolution error
IF OBJECTPROPERTY(object_id(N'tsql_notable'), 'IsProcedure') = 1
DROP PROCEDURE tsql_notable
go
CREATE PROCEDURE tsql_notable
AS
BEGIN TRY
 SELECT * from nonexistanttable
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError;  
END CATCH
go

execute tsql_notable

IF OBJECTPROPERTY(object_id(N'tsql_error_wrapper2'), 'IsProcedure') = 1
DROP PROCEDURE tsql_error_wrapper2
go
CREATE PROCEDURE tsql_error_wrapper2
AS
BEGIN TRY
 EXEC tsql_notable
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError;  
END CATCH
go

EXEC tsql_error_wrapper2

DROP PROCEDURE tsql_nested
go
CREATE PROCEDURE tsql_nested
AS
DECLARE @error_line INT
BEGIN TRY
  SET @error_line = 5; EXECUTE tsql_error_wrapper3
END TRY
BEGIN CATCH
  IF ERROR_PROCEDURE() = 'tsql_nested'
    SET @error_line = ERROR_LINE()
  EXECUTE usp_RethrowError2 'tsql_nested', @error_line
END CATCH
go

IF OBJECTPROPERTY(object_id(N'tsql_error_wrapper3'), 'IsProcedure') = 1
DROP PROCEDURE tsql_error_wrapper3
go
CREATE PROCEDURE tsql_error_wrapper3
AS 
DECLARE @error_line INT
BEGIN TRY
 SET @error_line = 5; EXEC tsql_trycatch1
END TRY
BEGIN CATCH
  IF ERROR_PROCEDURE() = 'tsql_error_wrapper3'
    SET @error_line = ERROR_LINE()
  EXECUTE usp_RethrowError2 'tsql_error_wrapper3', @error_line
END CATCH
go

EXECUTE tsql_nested

IF OBJECTPROPERTY(object_id(N'noparms'), 'IsProcedure') = 1
DROP PROCEDURE noparms
go

CREATE PROCEDURE noparms
AS
BEGIN
  PRINT 'In procedure noparms'
END
go


execute noparms 'someparm'
print 'after execute'
go

IF OBJECTPROPERTY(object_id(N'tsql_error_wrapper4'), 'IsProcedure') = 1
DROP PROCEDURE tsql_error_wrapper4
go
CREATE PROCEDURE tsql_error_wrapper4
AS
BEGIN TRY
 EXEC noparms 'someparm'
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError2 
END CATCH
go

EXECUTE tsql_error_wrapper4
SELECT @@error

-----------------------
-- SQLCLR errors

sp_addmessage 50001, 16, 'Error in SQLCLR proc'

EXEC clr_unhandled
SELECT @@ERROR

EXEC clr_rethrow
SELECT @@ERROR

EXEC clr_raiserror1
SELECT @@ERROR 

EXEC clr_raiserror2
SELECT @@ERROR 


IF OBJECTPROPERTY(object_id(N'error_wrapper_sqlclr1'), 'IsProcedure') = 1
DROP PROCEDURE error_wrapper_sqlclr1
go
CREATE PROCEDURE error_wrapper_sqlclr1
AS
BEGIN TRY
 EXEC clr_unhandled
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError;  
END CATCH

EXEC error_wrapper_sqlclr1
SELECT @@ERROR

IF OBJECTPROPERTY(object_id(N'error_wrapper_sqlclr2'), 'IsProcedure') = 1
DROP PROCEDURE error_wrapper_sqlclr2
go
CREATE PROCEDURE error_wrapper_sqlclr2
AS
BEGIN TRY
 EXEC clr_rethrow
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError;  
END CATCH
go

EXEC error_wrapper_sqlclr2
SELECT @@ERROR

IF OBJECTPROPERTY(object_id(N'error_wrapper_sqlclr3'), 'IsProcedure') = 1
DROP PROCEDURE error_wrapper_sqlclr3
go
CREATE PROCEDURE error_wrapper_sqlclr3
AS
BEGIN TRY
 EXEC clr_raiserror1
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError;  
END CATCH

EXEC error_wrapper_sqlclr3
SELECT @@ERROR 

IF OBJECTPROPERTY(object_id(N'error_wrapper_sqlclr4'), 'IsProcedure') = 1
DROP PROCEDURE error_wrapper_sqlclr4
go
CREATE PROCEDURE error_wrapper_sqlclr4
AS
BEGIN TRY
 EXEC clr_raiserror2
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError;  
END CATCH
go

EXEC error_wrapper_sqlclr4
SELECT @@ERROR

EXEC clr_nested_unhandled
SELECT @@ERROR

EXEC clr_pipe_error
SELECT @@ERROR

EXEC clr_pipe_dummy_catch
SELECT @@ERROR

BEGIN TRY
 EXEC clr_pipe_error
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError; 
END CATCH

BEGIN TRY
 EXEC clr_pipe_dummy_catch
END TRY
BEGIN CATCH
 EXECUTE usp_RethrowError; 
END CATCH

-- these work just like the unhandled ones
EXEC clr_wrapper
SELECT @@ERROR

EXEC clr_wrapper2
SELECT @@ERROR

IF OBJECTPROPERTY(object_id(N'tsql_dummy_wrapper'), 'IsProcedure') = 1
DROP PROCEDURE tsql_dummy_wrapper
go
CREATE PROCEDURE tsql_dummy_wrapper
AS
BEGIN TRY
 EXEC tsql_trycatch1
END TRY
BEGIN CATCH
 
END CATCH
go

-- test it, eats the error, no error message
EXEC tsql_dummy_wrapper
