/*
T-SQL error handling doesn't currently provide the capability to rethrow the
original error that caused execution to jump to the CATCH block. However,
error information is available in the scope of the CATCH block and RAISERROR
can be used to raise an error with the original error information embedded
in the message string. For example:
*/


BEGIN TRY
    SELECT 1/0;
END TRY
BEGIN CATCH
    DECLARE @ErrorMessage NVARCHAR(400);
    DECLARE @ErrorNumber INT;
    DECLARE @ErrorSeverity INT;
    DECLARE @ErrorState INT;
    DECLARE @ErrorLine INT;

    SELECT @ErrorMessage = N'Error %d, Line %d, Message: ' +
ERROR_MESSAGE();
    SELECT @ErrorNumber = ERROR_NUMBER();
    SELECT @ErrorSeverity = ERROR_SEVERITY();
    SELECT @ErrorState = ERROR_STATE();
    SELECT @ErrorLine = ERROR_LINE();
    RAISERROR (@ErrorMessage, @ErrorSeverity, @ErrorState, @ErrorNumber,
@ErrorLine)
END CATCH

Result:
Msg 50000, Level 16, State 1, Line 16
Error 8134, Line 2, Message: Divide by zero error encountered.
